#include <stdio.h>
#include "bhportable.h"

namespace bhpos{ 

CPortable::CPortable(BhDevType type) : CBhDevice(type)
{
}

CPortable::~CPortable()
{
}

int CPortable::OpenScanner(int device, BhDevNotifyType nt)
{
   dev=device;
}

int CPortable::GetNumBarcodes()
{
   return bc_list.size();
}

bool CPortable::InCradle()
{
   int readbit=0;
   
   /* poll for CTS?
   if(ioctl(dev, TIOCMGET, &readbit) == 0){
      return (readbit & TIOCM_CTS);
   }
   // simply return true for now*/
   return true;
}

int CPortable::PopBarcode(symbol_barcode *sbc)
{
   if(bc_list.empty()) return 0;

	std::list<symbol_barcode>::iterator iter=bc_list.begin();

	memcpy(sbc, &(*iter), sizeof(symbol_barcode));
	bc_list.pop_front();
	
   return 1;
}

// allow push back for bum codes
int CPortable::PushBarcode(symbol_barcode &sbc)
{
   bc_list.push_back(sbc);
   return bc_list.size();
}

int CPortable::LoadBarcodes()
{
   bool bMore=true;
   j=0;
   
   while((GetPacket() > 0) && bMore){
      if(pkt.opcode == BHPORTABLE_SYM_OPCODE_CMDNAK) {
         return -1;
      }
      else if(pkt.opcode == BHPORTABLE_SYM_OPCODE_BATCHDATA){
         if(pkt.status & BHPORTABLE_SYM_STATUS_LASTPACKET){
            return 0;
         }
         else{ // first packet
         }
      
         if(pkt.status & BHPORTABLE_SYM_STATUS_CONTPACKET){
            bMore=true;
         }
         else{ // not a continuation packet
            bMore=false;
         }
      }
   }
}

int CPortable::GetPacket()
{
   int i=0;
   unsigned short cksum_accum=0;
   
   pkt.length=FetchChar(); 
   cksum_accum -= pkt.length; 
   pkt.length--;
   
   pkt.opcode=FetchChar(); 
   cksum_accum -= pkt.opcode; 
   pkt.length--; 

   pkt.msg_src=FetchChar(); 
   cksum_accum -= pkt.msg_src; 
   pkt.length--;
  
   if(pkt.msg_src != BHPORTABLE_SYM_MSGSRC_DECODER){  // not the decoder
      return -1;
   }

   pkt.status=FetchChar();
   cksum_accum -= pkt.status; 
   pkt.length--;
   
   int state=0;
   int blen=0;
   
   while(pkt.length > 0) {
      pkt.msg[i]=FetchChar();
      cksum_accum-=pkt.msg[i]; 
      
      if(pkt.opcode==BHPORTABLE_SYM_OPCODE_BATCHDATA){
         switch(state){
         case 0: bc.size=pkt.msg[i]; state=1; blen=pkt.msg[i]; break;
         case 1: bc.type=pkt.msg[i]; state=2; break;
         case 2: bc.data[j]=pkt.msg[i]; j++; blen--; break;
         }
         if(blen==0){
            // store barcode until end of packet (checksum.)
            bc_list.push_back(bc);
            state=0;
            j=0;
            memset(&bc, 0, sizeof(bc));
         }
      }
      i++; 
      pkt.length--;
   }
   
   if(CheckSumOk(cksum_accum)) return 1;
   
   return -2;
}

int CPortable::BatchReq()
{
}

unsigned char CPortable::FetchChar()
{
   int            res;
   fd_set         fds;
   unsigned char  c;
   // need inter-char max timeout from spec here, but 20ms allows for a shit slow baud rate;
   struct timeval timeout={0, 20000}; 

   
   FD_ZERO(&fds);
   FD_SET(dev, &fds);
   
   res=select(dev+1, &fds, NULL, NULL, &timeout);
   if(res==0){ // timeout
      return 0;
   } 
   else if(res < 0){  // Error Occurred
      return 0;
   }
   read(dev, &c, 1);

   return c;
}

bool CPortable::CheckSumOk(unsigned short cksum_accum)
{
   unsigned short cksum;
   unsigned char  data;
   
   data=FetchChar();
   cksum = data << 8;
   data=FetchChar();
   cksum |= data;
   if(cksum != cksum_accum)
      return false;

   return true;
}

}
