/*
cprintermeh.cpp
Copyright (C)2005 Al Kinglsey

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Don't write to me at: kingsley@bananahead.com 

	Part of:
	BhPos, Copyright (C) 2001-2005 Al Kingsley
	BhPos comes with ABSOLUTELY NO WARRANTY.

*/

#include <cstring>
#include <fcntl.h>
#include <unistd.h>
#include <pthread.h>

#include "bhprinter.h"

namespace bhpos{

CPrintMechanism::CPrintMechanism()
{
	type=PM_TYPE_UNKNOWN;
	papertype=PM_PAPERTYPE_UNKNOWN;
	xpapersize=0;
	ypaperminsize=0;
	ypapermaxsize=0;
	xmotionunit=ymotionunit=0;	
	xdpi=ydpi=0;
	
	bHasPageMode=false;
	bBidrectional=false;
	cuttertype=PM_CUTTERTYPE_NONE;
	
	numcolours=1;
	curr_font=0;
}

CPrintMechanism::CPrintMechanism(CPrintMechanism const& pm)
{
	type=pm.type;
	xpapersize=pm.xpapersize;
	ypaperminsize=pm.ypaperminsize;
	ypapermaxsize=pm.ypapermaxsize;
	xmotionunit=pm.xmotionunit;
	ymotionunit=pm.ymotionunit;
	xdpi=pm.xdpi;
	ydpi=pm.ydpi;
	
	bHasPageMode=pm.bHasPageMode;
	bBidrectional=pm.bBidrectional;
	cuttertype=pm.cuttertype;
	numcolours=pm.numcolours;
	curr_font=pm.curr_font;
	
	fonts=pm.fonts;
}

CPrintMechanism::~CPrintMechanism()
{
	fonts.clear();
}

void CPrintMechanism::operator=(CPrintMechanism const& pm)
{
	type=pm.type;
	xpapersize=pm.xpapersize;
	ypaperminsize=pm.ypaperminsize;
	ypapermaxsize=pm.ypapermaxsize;
	xmotionunit=pm.xmotionunit;
	ymotionunit=pm.ymotionunit;
	xdpi=pm.xdpi;
	ydpi=pm.ydpi;
	
	bHasPageMode=pm.bHasPageMode;
	bBidrectional=pm.bBidrectional;
	cuttertype=pm.cuttertype;
	numcolours=pm.numcolours;
	curr_font=pm.curr_font;
	
	fonts=pm.fonts;
}

bool CPrintMechanism::operator==(CPrintMechanism const& pm)
{
	return false;
}

bool CPrintMechanism::operator<(CPrintMechanism const& pm)
{
	return false;
}

void CPrintMechanism::SetParameter(int param, void *v)
{
	switch(param){
	case PM_PARAM_TYPE:
		type=*((BhPrinterMechType *) v);
		break;
   case PM_PARAM_PAPERTYPE:
		papertype=*((BhPrinterMechPaperType *) v);
		break;
	case PM_PARAM_XMOTIONUNIT:
		xmotionunit=*((int *) v);
		break;
	case PM_PARAM_YMOTIONUNIT:
		ymotionunit=*((int *) v);
		break;
	case PM_PARAM_XPAPERSIZE:
		xpapersize=*((int *) v);
		break;
	case PM_PARAM_YPAPERMINSIZE:
		ypaperminsize=*((int *) v);
		break;
	case PM_PARAM_YPAPERMAXSIZE:
		ypapermaxsize=*((int *) v);
		break;
	case PM_PARAM_BIDIR:
		bBidrectional=*((bool *) v);
		break;
	case PM_PARAM_HASPAGEMODE:
		bHasPageMode=*((bool *) v);
		break;
	case PM_PARAM_CUTTER:
		cuttertype=*((BhPrinterMechCutterType *) v);
		break;
	case PM_PARAM_XDPI:
		xdpi=*((int *) v);
		break;
	case PM_PARAM_YDPI:
		ydpi=*((int *) v);
		break;
	case PM_PARAM_FONT:
		curr_font=*((int *) v);
		break;
	}
}

void CPrintMechanism::GetParameter(int param, void *v)
{
	switch(param){
	case PM_PARAM_TYPE:
		*((BhPrinterMechType *) v)=type;
		break;
   case PM_PARAM_PAPERTYPE:
		*((BhPrinterMechPaperType *) v)=papertype;
		break;
	case PM_PARAM_XMOTIONUNIT:
		*((int *) v)=xmotionunit;
		break;
	case PM_PARAM_YMOTIONUNIT:
		*((int *) v)=ymotionunit;
		break;
	case PM_PARAM_XPAPERSIZE:
		*((int *) v)=xpapersize;
		break;
	case PM_PARAM_YPAPERMINSIZE:
		ypaperminsize=*((int *) v);
		break;
	case PM_PARAM_YPAPERMAXSIZE:
		ypapermaxsize=*((int *) v);
		break;
	case PM_PARAM_BIDIR:
		bBidrectional=*((bool *) v);
		break;	
	case PM_PARAM_HASPAGEMODE:
		*((bool *) v)=bHasPageMode;
		break;
	case PM_PARAM_CUTTER:
		*((BhPrinterMechCutterType *) v)=cuttertype;
		break;
	case PM_PARAM_XDPI:
		*((int *) v)=xdpi;
		break;
	case PM_PARAM_YDPI:
		*((int *) v)=ydpi;
		break;
	case PM_PARAM_FONT:
		*((int *) v)=curr_font;
		break;
	case PM_PARAM_DEFCHARLINE:
	{
	   /*
	   list<CPrinterFont>::iterator fi=fonts.back();
	   if(curr_font > fonts.size() - 1) return;
	   int bob=curr_font;
	   while(bob){ fi--; bob--; }
	   fi->GetParameter(PF_PARAM_CHARLINE, v);*/
	}
	   break;
	}
}

int CPrintMechanism::AddFont(CPrinterFont& f)
{
	fonts.push_back(f);
	return 0;
}

}
