/*
cprinterparser.cpp
Copyright (C)2004 Al Kinglsey

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Don't write to me at: kingsley@bananahead.com 

	Part of:
	BhPos, Copyright (C) 2001-2005 Al Kingsley
	BhPos comes with ABSOLUTELY NO WARRANTY.
*/

#include <cstring>
#include <fcntl.h>
#include <unistd.h>
#include <pthread.h>

#include "bhprinter.h"

namespace bhpos{

namespace parser{

CPrinterDefinitionParser::CPrinterDefinitionParser()
{
   prnname="";
   manufacturer="";
   model="";
   protocol=ESCPOS;
   nvramsize=0;
   buffersize=0;
   bCustomerDisplay=false;
   //custdisp_type=0;
   //custdisp_x=0;
   //custdisp_y=0;
   //bCashDrawer=false;
   numcashdrawer=0;
   bMICR=false;
   bCutter=false;
}

CPrinterDefinitionParser::~CPrinterDefinitionParser()
{
}

int CPrinterDefinitionParser::Open(Glib::ustring &fname, CPrinter& prn)
{
   xmlpp::Node* node;
   Glib::ustring n;
   
   try{
      xmlpp::DomParser parser;
      parser.set_validate();
      parser.set_substitute_entities(); 
      parser.parse_file(fname);
      if(parser){
         nodeRoot = parser.get_document()->get_root_node(); 
         
      }
   }
   catch(const std::exception& ex){
      //std::cout << "Exception caught: " << ex.what() << std::endl;
   }
   
   if(const xmlpp::Element* nodeElement = dynamic_cast<const xmlpp::Element*>(nodeRoot)){
      const xmlpp::Attribute* attribute = nodeElement->get_attribute("name");
      if(attribute){
         prnname=attribute->get_value();
      }
   }
   xmlpp::Node::NodeList list = nodeRoot->get_children();
   for(xmlpp::Node::NodeList::iterator iter = list.begin(); iter!=list.end(); ++iter){
      node=*iter;
      n=node->get_name();
      if(n=="printer"){
         ParsePrinter(node, prn);
      }
      else if(n=="mechanism"){
         ParseMechanism(node, prn);
      }        
   }
}
   
int CPrinterDefinitionParser::ParsePrinter(xmlpp::Node *node, CPrinter& prn)
{   
   Glib::ustring           name, v;
   xmlpp::Node*            cnode;
   const xmlpp::Element*   nodeElement;
   const xmlpp::Attribute* attribute;
   
   xmlpp::Node::NodeList list = node->get_children();
   for(xmlpp::Node::NodeList::iterator iter = list.begin(); iter != list.end(); ++iter){
      cnode=*iter;
      name=cnode->get_name();
      if(name=="manufacturer"){
         if(nodeElement = dynamic_cast<const xmlpp::Element*>(cnode)){
            attribute = nodeElement->get_attribute("value");
            if(attribute){
               manufacturer=attribute->get_value();
            }
         }
      }
      else if(name=="model"){
         if(nodeElement = dynamic_cast<const xmlpp::Element*>(cnode)){
            attribute = nodeElement->get_attribute("value");
            if(attribute){
               model=attribute->get_value();
            }
         }
      }
      else if(name=="protocol"){
         if(nodeElement = dynamic_cast<const xmlpp::Element*>(cnode)){
            attribute = nodeElement->get_attribute("value");
            if(attribute){
               v=attribute->get_value();
               if(v=="escpos"){
                  protocol=ESCPOS;
               }
               else if(v=="dp8340"){
                  protocol=DP8340;
               }
               else if(v=="star"){
                  protocol=STAR;
               }
               else if(v=="citizen"){
                  protocol=CITIZEN;
               }
               else if(v=="pcos"){
                  protocol=PCOS;
               }
               else if(v=="axihom"){
                  protocol=AXIHOM;
               }
               else if(v=="pstemplate"){
                  protocol=PSTEMPLATE;
               }
            }
         }
      }
      else if(name=="nvram"){
         if(nodeElement = dynamic_cast<const xmlpp::Element*>(cnode)){
            attribute = nodeElement->get_attribute("value");
            if(attribute){
               v=attribute->get_value();
               nvramsize=atoi(v.data());
            }
         }
      }
      else if(name=="buffersize"){
         if(nodeElement = dynamic_cast<const xmlpp::Element*>(cnode)){
            attribute = nodeElement->get_attribute("value");
            if(attribute){
               v=attribute->get_value();
               buffersize=atoi(v.data());
            }
         }
      }
      else if(name=="numcashdrawer"){
         if(nodeElement = dynamic_cast<const xmlpp::Element*>(cnode)){
            attribute = nodeElement->get_attribute("value");
            if(attribute){
               v=attribute->get_value();
               numcashdrawer=atoi(v.data());
            }
         }
      }
      else if(name=="autocutter"){
         if(nodeElement = dynamic_cast<const xmlpp::Element*>(cnode)){
            attribute = nodeElement->get_attribute("value");
            if(attribute){
               v=attribute->get_value();
               v=v.lowercase();
               if(v=="y") bCutter=true;
               else bCutter=false;
            }
         }
      }
      else if(name=="customerdisplay"){
         if(nodeElement = dynamic_cast<const xmlpp::Element*>(cnode)){
            attribute = nodeElement->get_attribute("value");
            if(attribute){
               v=attribute->get_value();
               v=v.lowercase();
               if(v=="y") bCustomerDisplay=true;
               else bCustomerDisplay=false;
            }
         }
      }
      else if(name=="micr"){
         if(nodeElement = dynamic_cast<const xmlpp::Element*>(cnode)){
            attribute = nodeElement->get_attribute("value");
            if(attribute){
               v=attribute->get_value();
               v=v.lowercase();
               if(v=="y") bMICR=true;
               else bMICR=false;
            }
         }
      }
   }
   
   return 0;
}

// second level - recursive mech function
int CPrinterDefinitionParser::ParseMechanism(xmlpp::Node *node, CPrinter& prn)
{
   int                     i;
   Glib::ustring           name, v;
   xmlpp::Node*            cnode;
   const xmlpp::Element*   nodeElement;
   const xmlpp::Attribute* attribute;
   CPrintMechanism         mech;
   
   xmlpp::Node::NodeList list = node->get_children();
   for(xmlpp::Node::NodeList::iterator iter = list.begin(); iter != list.end(); ++iter){
      cnode=*iter;
      name=cnode->get_name();
      if(name=="manufacturer"){
         if(nodeElement = dynamic_cast<const xmlpp::Element*>(cnode)){
            attribute = nodeElement->get_attribute("type");
            if(attribute) v=attribute->get_value();
            mech.SetParameter(PM_PARAM_TYPE, (void *) v.data());
            
            attribute = nodeElement->get_attribute("papertype");
            if(attribute) v=attribute->get_value();
            i=atoi(v.data());
            mech.SetParameter(PM_PARAM_PAPERTYPE, &i);
            
            attribute = nodeElement->get_attribute("numfonts");
            if(attribute) v=attribute->get_value();
            i=atoi(v.data());
            mech.SetParameter(PM_PARAM_NUMFONTS, &i);
            
            attribute = nodeElement->get_attribute("numcols");
            if(attribute) v=attribute->get_value();
            i=atoi(v.data());
            mech.SetParameter(PM_PARAM_NUMCOLS, &i);
            
            attribute = nodeElement->get_attribute("x");
            if(attribute) v=attribute->get_value();
            i=atoi(v.data());
            mech.SetParameter(PM_PARAM_XPAPERSIZE, &i);
            
            attribute = nodeElement->get_attribute("ymin");
            if(attribute) v=attribute->get_value();
            i=atoi(v.data());
            mech.SetParameter(PM_PARAM_YPAPERMINSIZE, &i);
            
            attribute = nodeElement->get_attribute("ymax");
            if(attribute) v=attribute->get_value();
            i=atoi(v.data());
            mech.SetParameter(PM_PARAM_YPAPERMAXSIZE, &i);
            
            attribute = nodeElement->get_attribute("xmu");
            if(attribute) v=attribute->get_value();
            i=atoi(v.data());
            mech.SetParameter(PM_PARAM_XMOTIONUNIT, &i);
            
            attribute = nodeElement->get_attribute("ymu");
            if(attribute) v=attribute->get_value();
            i=atoi(v.data());
            mech.SetParameter(PM_PARAM_YMOTIONUNIT, &i);
            
            attribute = nodeElement->get_attribute("xdpi");
            if(attribute) v=attribute->get_value();
            i=atoi(v.data());
            mech.SetParameter(PM_PARAM_XDPI, &i);
            
            attribute = nodeElement->get_attribute("ydpi");
            if(attribute) v=attribute->get_value();
            i=atoi(v.data());
            mech.SetParameter(PM_PARAM_YDPI, &i);
         }
      }
   }
   /*		
      buff = xmlGetProp(node, (const xmlChar *) "bidir");	
      if(!xmlStrcmp(buff, (const xmlChar *) "Y") || !xmlStrcmp(buff, (const xmlChar *) "y"))
         b=true;
      else 
         b=false;
      mech.SetParameter(PM_PARAM_BIDIR, &b);
		
      buff = xmlGetProp(node, (const xmlChar *) "pagemode");
      if(!xmlStrcmp(buff, (const xmlChar *) "Y") || !xmlStrcmp(buff, (const xmlChar *) "y"))
         b=true;
      else 
         b=false;
      mech.SetParameter(PM_PARAM_HASPAGEMODE, &b);
		
      if((!xmlStrcmp(node->name, (const xmlChar *) "cutter")) && (node->ns==ns))
         ParseCutter(node, mech);
		
      if((!xmlStrcmp(node->name, (const xmlChar *) "colour")) && (node->ns==ns))
         ParseColour(node, mech);
		
      if((!xmlStrcmp(node->name, (const xmlChar *) "font")) && (node->ns==ns))
         ParseFont(node, mech);

      node=node->next;
	}
   */
   //prn.AddMechanism(mech);
}




int CPrinterDefinitionParser::ParseCutter(xmlpp::Node *node, CPrintMechanism& m)
{
	int 	  temp;
	xmlChar *name;
}

int CPrinterDefinitionParser::ParseColour(xmlpp::Node *node, CPrintMechanism& m)
{
   /*
	int 	  temp;
	xmlChar *name;
	
	node=node->xmlChildrenNode;

	while(node!=NULL){
		temp=atoi((const char *) xmlGetProp(node, (const xmlChar *) "type"));
		m.SetParameter(PM_PARAM_COLOUR, &temp);		
		node=node->next;
	}
	//m.AddColour();*/
}

int CPrinterDefinitionParser::ParseFont(xmlpp::Node *node, CPrintMechanism& m)
{
   int                     i;
   Glib::ustring           name, v;
   xmlpp::Node*            cnode;
   const xmlpp::Element*   nodeElement;
   const xmlpp::Attribute* attribute;
   CPrinterFont            font;
   
   xmlpp::Node::NodeList list = node->get_children();
   for(xmlpp::Node::NodeList::iterator iter = list.begin(); iter != list.end(); ++iter){
      cnode=*iter;
      name=cnode->get_name();
      if(name=="manufacturer"){
         if(nodeElement = dynamic_cast<const xmlpp::Element*>(cnode)){
            attribute = nodeElement->get_attribute("unit");
            if(attribute) v=attribute->get_value();
            i=atoi(v.data());
            font.SetParameter(PF_PARAM_UNIT, &i);
            
            attribute = nodeElement->get_attribute("xsize");
            if(attribute) v=attribute->get_value();
            i=atoi(v.data());
            font.SetParameter(PF_PARAM_XSIZE, &i);
            
            attribute = nodeElement->get_attribute("ysize");
            if(attribute) v=attribute->get_value();
            i=atoi(v.data());
            font.SetParameter(PF_PARAM_YSIZE, &i);
            
            attribute = nodeElement->get_attribute("charline");
            if(attribute) v=attribute->get_value();
            i=atoi(v.data());
            font.SetParameter(PF_PARAM_CHARLINE, &i);
         }
      }
   }
	m.AddFont(font);
   return 0;
}

}

}
