/*
cprntemplate.cpp
Copyright (C)2004 Al Kingsley

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   Don't write to me at: kingsley@bananahead.com 

   Part of:
   BhPos, Copyright (C) 2001-2005 Al Kingsley
   BhPos comes with ABSOLUTELY NO WARRANTY.
*/

#include "config.h"
#include <cstring>
#include <math.h>
#include <regex.h>

#include <libintl.h>
#define _(String) gettext(String)

#include "bhprinter.h"

namespace bhpos{

// should open dev something like this: popen("/usr/bin/lp -d bw_laser_raw","w");

int CPrinter::OpenTemplate(char *toppage, char *contpage, char *endpage)
{
   int len=0, maxlen=0;
   
   if(toppage!=NULL){
      tp=fopen(toppage, "r");
      if(tp==NULL) return 1;
      fseek(tp, 0, SEEK_END);
      len=ftell(tp);
      topbuff=new char[len+1];   
      if(!topbuff) return 2;
      top_len=len;
      fread(topbuff, len, 1, tp);
      GetPageVariables(1);
      tprinttype&=1;
      maxlen=len;
   }
   if(contpage!=NULL){
      cp=fopen(contpage, "r");
      if(cp==NULL) return 3;
      fseek(cp, 0, SEEK_END);
      len=ftell(cp);
      contbuff=new char[len+1];
      if(!contbuff) return 4;
      end_len=len;
      fread(contbuff, len, 1, cp);
      GetPageVariables(2);
      tprinttype&=2;
      if(len > maxlen) maxlen=len;
   }
   if(endpage!=NULL){
      ep=fopen(endpage, "r");
      if(ep==NULL) return 5;
      fseek(ep, 0, SEEK_END);
      len=ftell(ep);
      endbuff=new char[len+1];
      if(!endbuff) return 6;
      cont_len=len;
      fread(endbuff, len, 1, ep);
      GetPageVariables(3);
      tprinttype&=4;
      if(len > maxlen) maxlen=len;
   }
   
   // allow extra space to stop exceptionally dull mucho realloc'ing for tag 
   // replacements - though inefficient, it's a damn sight quicker just 
   // manipulating pointers.
   linebuffer=new char[maxlen+1024];
   if(!linebuffer) return 7;
   
   return 0;
}

int CPrinter::GetPageVariables(int f)
{
   FILE *fp;
   char  buff[64];
   char  tbuff[16];
   char *c;
   int   i=0, t=0;
   
   switch(f){
   case 1: fp=tp; break;
   case 2: fp=cp; break;
   case 3: fp=ep; break;
   }
   // parsing for %~ TAG ~%
   while(fgets(buff, 63, fp)){
      if(*buff){
         c=buff;
         while(*c!='%' || c!=NULL) c++;
         if(*c=='%'){
            c++;
            if(*c=='~'){
               while(*c!='='){
                  tbuff[i]=*c;
                  c++;
                  i++;
               }
               tbuff[i]=0;
               if(strcmp(tbuff, "yinc")==0){
                  t&=1;
               }
               if(strcmp(tbuff, "num")==0){
                  t&=2;
               }
               while(*c!='~'){
                  tbuff[i]=*c;
                  c++;
                  i++;
               }
               tbuff[i]=0;
               switch(f){
               case 1: 
                  if(t==1) top_yinc=atoi(tbuff);
                  else num_top=atoi(tbuff);
                  break;
               case 2: 
                  if(t==1) cont_yinc=atoi(tbuff);
                  else num_cont=atoi(tbuff);
                  break;
               case 3: 
                  if(t==1) end_yinc=atoi(tbuff);
                  else num_end=atoi(tbuff);
                  break;
               }
            }
         }
      }
      if(t&3) break;
   }
   return 0;
}

int CPrinter::PrintTemplate(CTransItems &trans, BhCustomerItem *cust, int numcopies)
{
   int s=0, c=0, e=0;
   
   if(tprinttype==1){         // top sheet only
      s=(int) ceil(trans.NumAllItems()/num_top);
     
      //ReplaceFixedPageTag();
   }
   else if(tprinttype==3){    // top & continuation sheet
      if(trans.NumAllItems() < num_top){
         s=1;
      }
      else{
         s=1;
         c=(int) ceil((trans.NumAllItems() - num_top) / num_cont);
      }
      
   }
   else if(tprinttype==5){    // top & end sheet
      if(trans.NumAllItems() < num_top){
         s=1;
      }
      else{
         s=1;
         e=(int) ceil((trans.NumAllItems() - num_top) / num_end);
      }
   }
   else if(tprinttype==7){    // top, continuation and end sheets
      if(trans.NumAllItems() < num_top){
         s=1;
      }
      else{
         if((trans.NumAllItems() - num_top) > num_end){
            s=1;
            c=(int) floor(trans.NumAllItems() - num_top)/num_cont;
            e=trans.NumAllItems() - ((s * num_top) + (c * num_cont));
         }
         else{
            s=1;
            e=1;
         }
      }
   }
   
   return 0;
}

int CPrinter::ReplaceTag(int f, char *tag, char *rep)
{
   char *m, *c=linebuffer;
   char  tbuff[128];
   int   i=0, z=0, flen=0;
   
   switch(f){
   case 1: flen=top_len; break;
   case 2: flen=cont_len; break;
   case 3: flen=end_len; break;
   }
   
   while(*c!=0){
      while(*c!='%' || c!=NULL) c++;
      if(*c=='%'){
         m=c;
         c++;
         if(*c=='%'){
            c++;
            while(*c!='%'){
               tbuff[i]=*c;
               c++;
               i++;
            }
            c++;
            if(*c!='%') continue;
            tbuff[i]=0;
            if(strcmp(tbuff, tag)==0){
               // replace the tag
               z=i-strlen(rep);
               if(z > 0){  // if the replacement is longer than the tag, then we need to make space
                  memcpy((m+i+1), (m+strlen(rep)), flen-((&m-&linebuffer)+strlen(rep)));
                  memcpy(m, rep, strlen(rep));
               }
               else if (z < 0){ // or reduce
                  memcpy(m, rep, strlen(rep));
                  while(z!=0){ 
                     *c=0x20;
                     c++;
                     z++;
                  }
               }
               else{ // or an unlikely exact length
                  memcpy(m, rep, strlen(rep));
               }
               
               return 0;
            }
            else
               continue;
         }
      }
   }
   return 1;
}

int CPrinter::ReplaceFixedInstances(int f, int numc, int sheetnum, int sheetof, CTransItems &trans, BhCustomerItem *cust)
{
   int  n=numc;
   char buffler[32], day[16], month[16], year[8], time[16];
   
   while(n){
      // sheet date/time
      ReplaceTag(f, "%%date_day%%", day);
      ReplaceTag(f, "%%date_month%%", month);
      ReplaceTag(f, "%%date_year%%", year);
      ReplaceTag(f, "%%time_time%%", time);
      
      // sheet stuff
      sprintf(buffler, "%d", sheetnum);
      ReplaceTag(f, "%%sheet_sheetnum%%", buffler);
      sprintf(buffler, "%d", sheetof);
      ReplaceTag(f, "%%sheet_sheetof%%", buffler);      
         
      // sheet customer
      if(cust){
         ReplaceTag(f, "%%cust_fname%%", cust->fname);
         ReplaceTag(f, "%%cust_lname%%", cust->lname);
         ReplaceTag(f, "%%cust_address1%%", cust->address1);
         ReplaceTag(f, "%%cust_address2%%", cust->address2);
         ReplaceTag(f, "%%cust_address3%%", cust->address3);
         ReplaceTag(f, "%%cust_city%%", cust->city);
         ReplaceTag(f, "%%cust_state%%", cust->state);
         ReplaceTag(f, "%%cust_postcode%%", cust->zip);
      }
         
      // sheet trans
      sprintf(buffler, "%012d", trans.GetID());
      ReplaceTag(f, "%%trans_ID%%", buffler);
      sprintf(buffler, "%d", trans.NumAllItems());
      ReplaceTag(f, "%%trans_numitems%%", buffler);
      sprintf(buffler, "%d", trans.NumAllItemsQty());
      ReplaceTag(f, "%%trans_numitemsqty%%", buffler);
      //sprintf(buffler, "%0.2f", trans.GetSubTotal());
      //ReplaceTag(f, "%%trans_subtotal%%", buffler);
      //sprintf(buffler, "%0.2f", trans.GetTotal());
      //ReplaceTag(f, "%%trans_total%%", buffler);
      //sprintf(buffler, "%0.2f", trans.GetTax());
      //ReplaceTag(f, "%%trans_tax%%", buffler);
         
      n--;
   }
}

int CPrinter::ReplaceVariableInstances(int f, int numc, CSaleItem &si)
{
   int n=numc;
   
   while(n){
      /*
      ReplaceTag(f, "%%item_plu%%", si.GetPLU());
      ReplaceTag(f, "%%item_qty%%", si.GetQuantity());
      ReplaceTag(f, "%%item_description%%", si.GetRecText());
      ReplaceTag(f, "%%item_price%%", si.GetPrice());
      ReplaceTag(f, "%%item_subtotal%%", si.GetTotal());
      */
      
      //ReplaceTag(f, "%%item_tax%%", si.GetTax());
      //ReplaceTag(f, "%%item_total%%", si.GetTotal());
      
      n--;
   }
   return 0;
}

/*
int CPrinter::ReplaceInstances(int f, CTransItems &trans, BhCustomerItem *cust)
{
   int     n=num_copies;
   regex_t reg;
   
   while(num_sheets){
      while(n){
         // sheet date/time
         if(!regcomp(&reg, "s/%%date_day%%/cust->fname/g", 0)){
            if(!regexec(&reg, linebuffer, 0, NULL, 0)){
            }
            regfree(&reg);
         }
         if(!regcomp(&reg, "s/%%date_month%%/cust->fname/g", 0)){
            if(!regexec(&reg, linebuffer, 0, NULL, 0)){
            }
            regfree(&reg);
         }
         if(!regcomp(&reg, "s/%%date_year%%/cust->fname/g", 0)){
            if(!regexec(&reg, linebuffer, 0, NULL, 0)){
            }
            regfree(&reg);
         }
         if(!regcomp(&reg, "s/%%time_time%%/cust->fname/g", 0)){
            if(!regexec(&reg, linebuffer, 0, NULL, 0)){
            }
            regfree(&reg);
         }
         
         // sheet numbering
         if(!regcomp(&reg, "s/%%sheet_sheetnum%%/cust->fname/g", 0)){
            if(!regexec(&reg, linebuffer, 0, NULL, 0)){
            }
            regfree(&reg);
         }
         if(!regcomp(&reg, "s/%%sheet_sheetof%%/cust->fname/g", 0)){
            if(!regexec(&reg, linebuffer, 0, NULL, 0)){
            }
            regfree(&reg);
         }
         
         // sheet customer
         if(!regcomp(&reg, "s/%%cust_fname%%/cust->fname/g", 0)){
            if(!regexec(&reg, linebuffer, 0, NULL, 0)){
            }
            regfree(&reg);
         }
         if(!regcomp(&reg, "s/%%cust_lname%%/cust->lname/g", 0)){
            if(!regexec(&reg, linebuffer, 0, NULL, 0)){
            }
            regfree(&reg);
         }
         if(!regcomp(&reg, "s/%%cust_address1%%/cust->address1/g", 0)){
            if(!regexec(&reg, linebuffer, 0, NULL, 0)){
            }
            regfree(&reg);
         }
         if(!regcomp(&reg, "s/%%cust_address2%%/cust->address2/g", 0)){
            if(!regexec(&reg, linebuffer, 0, NULL, 0)){
            }
            regfree(&reg);
         }
         if(!regcomp(&reg, "s/%%cust_address3%%/cust->address3/g", 0)){
            if(!regexec(&reg, linebuffer, 0, NULL, 0)){
            }
            regfree(&reg);
         }
         if(!regcomp(&reg, "s/%%cust_city%%/cust->city/g", 0)){
            if(!regexec(&reg, linebuffer, 0, NULL, 0)){
            }
            regfree(&reg);
         }
         if(!regcomp(&reg, "s/%%cust_state%%/cust->state/g", 0)){
            if(!regexec(&reg, linebuffer, 0, NULL, 0)){
            }
            regfree(&reg);
         }
         if(!regcomp(&reg, "s/%%cust_postcode%%/cust->postcode/g", 0)){
            if(!regexec(&reg, linebuffer, 0, NULL, 0)){
            }
            regfree(&reg);
         }
         
         // sheet trans
         if(!regcomp(&reg, "s/%%trans_ID%%/trans.GetID()/g", 0)){
            if(!regexec(&reg, linebuffer, 0, NULL, 0)){
            }
            regfree(&reg);
         }
         if(!regcomp(&reg, "s/%%trans_numitems%%/trans.GetID()/g", 0)){
            if(!regexec(&reg, linebuffer, 0, NULL, 0)){
            }
            regfree(&reg);
         }
         if(!regcomp(&reg, "s/%%trans_subtotal%%/trans.GetID()/g", 0)){
            if(!regexec(&reg, linebuffer, 0, NULL, 0)){
            }
            regfree(&reg);
         }
         if(!regcomp(&reg, "s/%%trans_total%%/trans.GetID()/g", 0)){
            if(!regexec(&reg, linebuffer, 0, NULL, 0)){
            }
            regfree(&reg);
         }
         if(!regcomp(&reg, "s/%%trans_tax%%/trans.GetID()/g", 0)){
            if(!regexec(&reg, linebuffer, 0, NULL, 0)){
            }
            regfree(&reg);
         }
         
         // sheet items
         while(42){
            if(!regcomp(&reg, "s/%%item_plu%%/plu/g", 0)){
               if(!regexec(&reg, linebuffer, 0, NULL, 0)){
               }
               regfree(&reg);
            }
            if(!regcomp(&reg, "s/%%item_qty%%/plu/g", 0)){
               if(!regexec(&reg, linebuffer, 0, NULL, 0)){
               }
               regfree(&reg);
            }
            if(!regcomp(&reg, "s/%%item_description%%/plu/g", 0)){
               if(!regexec(&reg, linebuffer, 0, NULL, 0)){
               }
               regfree(&reg);
            }
            if(!regcomp(&reg, "s/%%item_price%%/plu/g", 0)){
               if(!regexec(&reg, linebuffer, 0, NULL, 0)){
               }
               regfree(&reg);
            }
            if(!regcomp(&reg, "s/%%item_subtotal%%/plu/g", 0)){
               if(!regexec(&reg, linebuffer, 0, NULL, 0)){
               }
               regfree(&reg);
            }
            if(!regcomp(&reg, "s/%%item_tax%%/plu/g", 0)){
               if(!regexec(&reg, linebuffer, 0, NULL, 0)){
               }
               regfree(&reg);
            }
            if(!regcomp(&reg, "s/%%item_total%%/plu/g", 0)){
               if(!regexec(&reg, linebuffer, 0, NULL, 0)){
               }
               regfree(&reg);
            }
         }
         n--;
      }
      n=num_copies;
      num_sheets--;
   }
}
*/
}
